//
//  SkyDriveExporter.m
//  SkyDriveExporter.iPhotoExporter
//
//  Created by Eugene Solodovnykov on 5/15/12.
//  Copyright (c) 2012. All rights reserved.
//

#import "SkyDriveExporter.h"
#import "AFNetworking.h"

@interface SkyDriveExporter () <ExportPluginProtocol, NSWindowDelegate> 

@property (nonatomic, retain) NSLock *progressLock;

@property (nonatomic, retain) NSOperationQueue *operationQueue;
@property (nonatomic, copy) NSString *accessToken;
@property (nonatomic, copy) NSString *uploadLocation;

- (void)proceedUpload;
- (void)finalizeUploadIfNeeded;

@end

@implementation SkyDriveExporter

@synthesize progressLock = _progressLock;
@synthesize operationQueue = _operationQueue;
@synthesize accessToken = _accessToken;
@synthesize uploadLocation = _uploadLocation;

- (void)dealloc {
    self.progressLock = nil;
    self.operationQueue = nil;
    self.accessToken = nil;
    self.uploadLocation = nil;

    [super dealloc];
}

//here's the original implementation
- (id)initWithExportImageObj:(ExportMgr *)exportMgr {
    self = [super init];
    
    if (self) {
        _exportMgr = exportMgr;
        [self lockProgress];
        _progress.message = nil;
        _progress.indeterminateProgress = YES;
        [self unlockProgress];
    }

    return self;
}

//seems like that should be NSBox<SkyDriveExporterPluginBox>
- (id)settingsView {
    return _settingsBox;
}

//it's the view which would be selected on tab's activation
- (id)firstView {
    return _firstBox;
}

//some preparations could be done here
- (void)viewWillBeActivated {
    [_exportMgr enableControls];
}

//clean the view
//if it's needed
- (void)viewWillBeDeactivated {
    
}

//QTExporter returns @"mov"
//HTMLExporter returns empty strings
//FileExporter returns empty string for multiple selection (more then one image has been selected to export)
//  otherwise – it would use some function to get the fileName
//
//  if ([_exportMgr imageCount] == 1) {
//      return [[self _destinationFilename:_exportMgr] pathExtension]; //-(id)_destinationFilename:(id)arg1
//  }
//  return @"";
//
//Empty string indicates that fileType shouldn't be added to a fileName
- (NSString *)requiredFileType {
    return @"";
}

//says iPhoto whether it needs to present destination prompt selection dialog or no
- (BOOL)wantsDestinationPrompt {
    return NO;
}

//it returns empty string for all checked plugins
//probably it's just a legacy method
- (NSString *)getDestinationPath {
    return @"";
}

//that filename would be prefilled in a save dialog if wantsDestinationPrompt would return YES
- (NSString *)defaultFileName {
    return @"";
}

//that directory would be shouwn by default in a destination path selecting dialog
//NOTE: it should end with trailing slash, i.e. @"~/Desktop/"
//this method would be called even if you'd return NO from wantsDestinationPrompt:
//default path is @"~/Pictures/"
//
//it's possible to use Localizable.strings file to provive that value
//in that case it's needed to specify bundle:
//  NSBundle *bundle = [NSBundle bundleForClass:[SkyDriveExporter class]];
//  return NSLocalizedStringFromTableInBundle(@"DefaultDirectory", nil, bundle, nil);
- (NSString *)defaultDirectory {
    return nil;
}

//in case wantsDestinationPrompt returns YES and multiple items are selected to export:
//  - would present file selection dialog if that method returns NO
//  - would present directory selection dialog if that method returns YES
//
//method wouldn't be called if wantsDestinationPrompt returns NO
- (BOOL)treatSingleSelectionDifferently {
    return NO;
}

//says iPhoto if the plugin supports movie files exporting
- (BOOL)handlesMovieFiles {
    return NO;
}

//I haven't got that method to be called :(
//FileExporter returns NO
//HTMLExporter returns YES and does some unimportant checks
//QTExporter returns NO
- (BOOL)validateUserCreatedPath:(id)arg1 {
    return NO;
}

- (void)clickExport {
    [_exportMgr clickExport];
}

//it's possible to do some preparations here
//once [_exportMgr startExport]; would be called – iPhoto would show progress indicator
//method would be called in a main thread
- (void)startExport:(NSString *)path {
    [_authWebView setMainFrameURL:@"https://login.live.com/oauth20_authorize.srf?client_id=000000004C09FB95&scope=wl.skydrive_update&response_type=token&redirect_uri=https://login.live.com/oauth20_desktop.srf"];
    [_authWebView setPolicyDelegate:self];

    [_authWindow makeKeyWindow];
    NSInteger retValue = [NSApp runModalForWindow:_authWindow];
    [_authWindow close];

    if (retValue == errAuthorizationSuccess) {
        [_exportMgr startExport];
    }
}

//main exporting routine
//method would be called in a background thread
- (void)performExport:(NSString *)path {
    NSMutableURLRequest *albumCreationRequest = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:@"https://apis.live.net/v5.0/me/albums"]];
    [albumCreationRequest setHTTPMethod:@"POST"];
    [albumCreationRequest addValue:@"application/json" forHTTPHeaderField:@"Content-Type"];
    [albumCreationRequest addValue:[NSString stringWithFormat:@"Bearer %@", self.accessToken] forHTTPHeaderField:@"Authorization"];
    [albumCreationRequest setHTTPBody:[@"{name: \"iPhoto\"}" dataUsingEncoding:NSUTF8StringEncoding]];

    AFJSONRequestOperation *albumCreationOperation = [AFJSONRequestOperation JSONRequestOperationWithRequest:albumCreationRequest success:^(NSURLRequest *request, NSHTTPURLResponse *response, id JSON) {
        self.uploadLocation = [JSON objectForKey:@"upload_location"];
        [self proceedUpload];
    } failure:^(NSURLRequest *request, NSHTTPURLResponse *response, NSError *error, id JSON) {
        if (response.statusCode == 400 && [[JSON valueForKeyPath:@"error.code"] isEqualToString:@"resource_already_exists"]) {
            NSString *albumsListURLString = [NSString stringWithFormat:@"https://apis.live.net/v5.0/me/albums?access_token=%@", self.accessToken];
            NSMutableURLRequest *albumsListRequest = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:albumsListURLString]];
            [albumsListRequest addValue:@"application/json" forHTTPHeaderField:@"Accept"];
            
            AFJSONRequestOperation *albumsListOperation = [AFJSONRequestOperation JSONRequestOperationWithRequest:albumsListRequest success:^(NSURLRequest *request, NSHTTPURLResponse *response, id JSON) {
                NSPredicate *predicate = [NSPredicate predicateWithFormat:@"%K == %@", @"name", @"iPhoto"];
                NSArray *albumArray = [[JSON objectForKey:@"data"] filteredArrayUsingPredicate:predicate];
                if ([albumArray count]) {
                    self.uploadLocation = [[albumArray objectAtIndex:0] objectForKey:@"upload_location"];
                    [self proceedUpload];
                }
            } failure:^(NSURLRequest *request, NSHTTPURLResponse *response, NSError *error, id JSON) {
                //TODO
            }];
            
            [self.operationQueue addOperation:albumsListOperation];
        }
    }];

    [self.operationQueue addOperation:albumCreationOperation];

//    //here's the part which would close exporting window
//    //it's needed to use locks because that (or any other code for _progress) might be called from the background (?)
//    [self lockProgress];
//    _progress.message = nil; //it's needed to do so because if exporting was cancelled and started again – progress popup would still say "Cancelling..."
//    _progress.shouldStop = YES; //closes exporting window
//    [self unlockProgress];
}

//just returns instance variable's pointer
- (ExportProgress *)progress {
    return &_progress;
}

- (void)lockProgress {
    [self.progressLock lock];
}

- (void)unlockProgress {
    [self.progressLock unlock];
}

//just set some indicator which would be checked in -performExport:
- (void)cancelExport {
    [self.operationQueue cancelAllOperations];

    [self lockProgress];
    _progress.shouldCancel = YES;
    [self unlockProgress];
}

//usually returns plugin's binary name but can be any (even empty or just nil) string
- (id)name {
    return @"SkyDriveExporter";
}

#pragma mark -

- (void)proceedUpload {
    [self lockProgress];
    _progress.totalItems = _exportMgr.imageCount;
    _progress.currentItem = 0;
    [self unlockProgress];

    AFHTTPClient *httpClient = [[AFHTTPClient alloc] initWithBaseURL:[NSURL URLWithString:self.uploadLocation]];
    
    for (NSUInteger imageIndex = 0; imageIndex < _exportMgr.imageCount; imageIndex++) {
        NSString *imagePath = [_exportMgr imagePathAtIndex:imageIndex];
        NSString *imageName = [imagePath lastPathComponent];
        NSString *encodedImageName = (NSString *)CFURLCreateStringByAddingPercentEscapes(NULL, (CFStringRef)imageName, NULL, (CFStringRef)@"!*'();:@&=+$,/?%#[]", kCFStringEncodingUTF8);

        NSString *path = [encodedImageName stringByAppendingFormat:@"?access_token=%@", self.accessToken];
        NSURL *url = [NSURL URLWithString:path relativeToURL:[NSURL URLWithString:self.uploadLocation]];

        NSMutableURLRequest *fileUploadRequest = [NSMutableURLRequest requestWithURL:url];
        [fileUploadRequest setHTTPMethod:@"PUT"];
        [fileUploadRequest setValue:@"application/json" forHTTPHeaderField:@"Accept"];
        [fileUploadRequest setHTTPBody:[NSData dataWithContentsOfFile:imagePath]];

        AFJSONRequestOperation *fileUploadOperation = [AFJSONRequestOperation JSONRequestOperationWithRequest:fileUploadRequest success:^(NSURLRequest *request, NSHTTPURLResponse *response, id JSON) {
            [self finalizeUploadIfNeeded];
        } failure:^(NSURLRequest *request, NSHTTPURLResponse *response, NSError *error, id JSON) {
            //TODO
            [self finalizeUploadIfNeeded];
        }];

        [self.operationQueue addOperation:fileUploadOperation];
    }

    [httpClient release];
}

- (void)finalizeUploadIfNeeded {
    [self lockProgress];
    _progress.currentItem += 1;
    
    if (_progress.currentItem == _progress.totalItems) {
        _progress.shouldStop = YES;
    }
    [self unlockProgress];
}

- (NSLock *)progressLock {
    if (!_progressLock) {
        _progressLock = [[NSLock alloc] init];
    }
    return _progressLock;
}

- (NSOperationQueue *)operationQueue {
    if (!_operationQueue) {
        _operationQueue = [[NSOperationQueue alloc] init];
        [_operationQueue setMaxConcurrentOperationCount:1];
    }
    return _operationQueue;
}

#pragma mark - NSWindowDelegate

- (void)webView:(WebView *)sender decidePolicyForNavigationAction:(NSDictionary *)actionInformation request:(NSURLRequest *)request frame:(WebFrame *)frame decisionListener:(id<WebPolicyDecisionListener>)listener {
    NSURL *url = [request URL];
    
    if ([[url host] isEqualToString:@"login.live.com"] && [[url path] isEqualToString:@"/oauth20_desktop.srf"]) {
        NSInteger authorizationCode = errAuthorizationDenied;

        NSArray *fragmentComponents = [[url fragment] componentsSeparatedByString:@"&"];
        for (NSString *component  in fragmentComponents) {
            NSArray *componentParts = [component componentsSeparatedByString:@"="];
            if ([[componentParts objectAtIndex:0] isEqualToString:@"access_token"]) {
                self.accessToken = [componentParts objectAtIndex:1];
                authorizationCode = errAuthorizationSuccess;
                break;
            }
            if ([[componentParts objectAtIndex:0] isEqualToString:@"error"]) {
                break;
            }
        }
        [[NSApplication sharedApplication] stopModalWithCode:authorizationCode];
    } else {
        [listener use];
    }
}

- (void)windowWillClose:(NSNotification *)notification {
    [[NSApplication sharedApplication] stopModalWithCode:errAuthorizationCanceled];
}

@end
